summaryrefslogtreecommitdiff
path: root/libpod/container_api.go
diff options
context:
space:
mode:
authorMatthew Heon <matthew.heon@pm.me>2019-12-12 16:19:36 -0500
committerMatthew Heon <matthew.heon@pm.me>2019-12-12 16:35:37 -0500
commitbd44fd5c815fc750fd6b60899328564bee74e6e5 (patch)
tree980a0e85eec4f3a646c1f0ac63c05634fd13510b /libpod/container_api.go
parentc2dab75f0ec1502e075cf5174d1b308267648fda (diff)
downloadpodman-bd44fd5c815fc750fd6b60899328564bee74e6e5.tar.gz
podman-bd44fd5c815fc750fd6b60899328564bee74e6e5.tar.bz2
podman-bd44fd5c815fc750fd6b60899328564bee74e6e5.zip
Reap exec sessions on cleanup and removal
We currently rely on exec sessions being removed from the state by the Exec() API itself, on detecting the session stopping. This is not a reliable method, though. The Podman frontend for exec could be killed before the session ended, or another Podman process could be holding the lock and prevent update (most notable in `run --rm`, when a container with an active exec session is stopped). To resolve this, add a function to reap active exec sessions from the state, and use it on cleanup (to clear sessions after the container stops) and remove (to do the same when --rm is passed). This is a bit more complicated than it ought to be because Kata and company exist, and we can't guarantee the exec session has a PID on the host, so we have to plumb this through to the OCI runtime. Fixes #4666 Signed-off-by: Matthew Heon <matthew.heon@pm.me>
Diffstat (limited to 'libpod/container_api.go')
-rw-r--r--libpod/container_api.go7
1 files changed, 6 insertions, 1 deletions
diff --git a/libpod/container_api.go b/libpod/container_api.go
index 153a1d628..5168dbc68 100644
--- a/libpod/container_api.go
+++ b/libpod/container_api.go
@@ -594,7 +594,12 @@ func (c *Container) Cleanup(ctx context.Context) error {
// If we didn't restart, we perform a normal cleanup
- // Check if we have active exec sessions
+ // Reap exec sessions first.
+ if err := c.reapExecSessions(); err != nil {
+ return err
+ }
+
+ // Check if we have active exec sessions after reaping.
if len(c.state.ExecSessions) != 0 {
return errors.Wrapf(define.ErrCtrStateInvalid, "container %s has active exec sessions, refusing to clean up", c.ID())
}